import requests
import os
import json
from typing import Dict, Any, Optional


class HostNotFoundError(Exception):
    def __init__(self, message="Host not found"):
        super().__init__(message)


class HivetraceSDK:
    def __init__(self, config_path: str) -> None:
        self.hivetrace_url = self._load_hivetrace_url(config_path)
        self.session = requests.Session()

    def _load_hivetrace_url(self, config_path: str) -> str:
        if not os.path.exists(config_path):
            raise FileNotFoundError(f"File {config_path} not found")

        with open(config_path, "r") as file:
            config = json.load(file)

        hivetrace_url = config.get("hivetrace_url")
        if not hivetrace_url:
            raise HostNotFoundError()

        return hivetrace_url.rstrip("/")

    def _send_request(
        self,
        endpoint: str,
        application_id: str,
        message: str,
        additional_parameters: Optional[Dict[str, Any]],
    ) -> Dict[str, Any]:
        url = f"{self.hivetrace_url}{endpoint}"
        payload = {
            "application_id": application_id,
            "message": message,
            "additional_parameters": additional_parameters or {},
        }
        response = self.session.post(url, json=payload)
        return response

    def monitor_user(
        self,
        application_id: str,
        message: str,
        additional_parameters: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        return self._send_request(
            "/process_request/", application_id, message, additional_parameters
        )

    def monitor_llm(
        self,
        application_id: str,
        message: str,
        additional_parameters: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        return self._send_request(
            "/process_response/", application_id, message, additional_parameters
        )

    def __del__(self) -> None:
        if hasattr(self, "session"):
            self.session.close()
